@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2021, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -30,18 +30,25 @@
 """
 
 import ssl
+import warnings
 
+from .utils import make_abc
 from .errors import ProgrammingError
 from .charsets import MYSQL_CHARACTER_SETS
 
 MAX_PACKET_LENGTH = 16777215
 NET_BUFFER_LENGTH = 8192
 MAX_MYSQL_TABLE_COLUMNS = 4096
+# Flag used to send the Query Attributes with 0 (or more) parameters.
+PARAMETER_COUNT_AVAILABLE = 8
 
 DEFAULT_CONFIGURATION = {
     'database': None,
     'user': '',
     'password': '',
+    'password1': '',
+    'password2': '',
+    'password3': '',
     'host': '127.0.0.1',
     'port': 3306,
     'unix_socket': None,
@@ -49,6 +56,7 @@ DEFAULT_CONFIGURATION = {
     'charset': 'utf8mb4',
     'collation': None,
     'converter_class': None,
+    'converter_str_fallback': False,
     'autocommit': False,
     'time_zone': None,
     'sql_mode': None,
@@ -79,7 +87,9 @@ DEFAULT_CONFIGURATION = {
     'consume_results': False,
     'conn_attrs': None,
     'dns_srv': False,
-    'use_pure': False
+    'use_pure': False,
+    'krb_service_principal': None,
+    'oci_config_file': None
 }
 
 CNX_POOL_ARGS = ('pool_name', 'pool_size', 'pool_reset_session')
@@ -96,6 +106,12 @@ def flag_is_set(flag, flags):
     return False
 
 
+def _obsolete_option(name, new_name, value):
+    warnings.warn('The option "{}" has been deprecated, use "{}" instead.'
+                  ''.format(name, new_name), category=DeprecationWarning)
+    return value
+
+
 class _Constants(object):
     """
     Base class for constants
@@ -337,7 +353,7 @@ class ServerCmd(_Constants):
     BINLOG_DUMP = 18
     TABLE_DUMP = 19
     CONNECT_OUT = 20
-    REGISTER_SLAVE = 21
+    REGISTER_REPLICA = 21
     STMT_PREPARE = 22
     STMT_EXECUTE = 23
     STMT_SEND_LONG_DATA = 24
@@ -371,7 +387,7 @@ class ServerCmd(_Constants):
         'BINLOG_DUMP': (18, 'BINLOG_DUMP'),
         'TABLE_DUMP': (19, 'TABLE_DUMP'),
         'CONNECT_OUT': (20, 'CONNECT_OUT'),
-        'REGISTER_SLAVE': (21, 'REGISTER_SLAVE'),
+        'REGISTER_REPLICA': (21, 'REGISTER_REPLICA'),
         'STMT_PREPARE': (22, 'STMT_PREPARE'),
         'STMT_EXECUTE': (23, 'STMT_EXECUTE'),
         'STMT_SEND_LONG_DATA': (24, 'STMT_SEND_LONG_DATA'),
@@ -415,6 +431,7 @@ class ClientFlag(_Flags):
     CAN_HANDLE_EXPIRED_PASSWORDS = 1 << 22
     SESION_TRACK = 1 << 23
     DEPRECATE_EOF = 1 << 24
+    CLIENT_QUERY_ATTRIBUTES = 1 << 27
     SSL_VERIFY_SERVER_CERT = 1 << 30
     REMEMBER_OPTIONS = 1 << 31
 
@@ -445,6 +462,7 @@ class ClientFlag(_Flags):
         'CAN_HANDLE_EXPIRED_PASSWORDS': (1 << 22, "Don't close the connection for a connection with expired password"),
         'SESION_TRACK': (1 << 23, 'Capable of handling server state change information'),
         'DEPRECATE_EOF': (1 << 24, 'Client no longer needs EOF packet'),
+        'CLIENT_QUERY_ATTRIBUTES': (1 << 27, 'Support optional extension for query parameters'),
         'SSL_VERIFY_SERVER_CERT': (1 << 30, ''),
         'REMEMBER_OPTIONS': (1 << 31, ''),
     }
@@ -458,7 +476,7 @@ class ClientFlag(_Flags):
         SECURE_CONNECTION,
         MULTI_STATEMENTS,
         MULTI_RESULTS,
-        CONNECT_ARGS
+        CONNECT_ARGS,
     ]
 
     @classmethod
@@ -531,6 +549,13 @@ class ServerFlag(_Flags):
     }
 
 
+class RefreshOption_meta(type):
+    @property
+    def SLAVE(self):
+        return _obsolete_option("RefreshOption.SLAVE", "RefreshOption.REPLICA",
+                                RefreshOption.REPLICA)
+
+@make_abc(RefreshOption_meta)
 class RefreshOption(_Constants):
     """MySQL Refresh command options
 
@@ -543,16 +568,17 @@ class RefreshOption(_Constants):
     HOST = 1 << 3
     STATUS = 1 << 4
     THREADS = 1 << 5
-    SLAVE = 1 << 6
+    REPLICA = 1 << 6
 
     desc = {
         'GRANT': (1 << 0, 'Refresh grant tables'),
         'LOG': (1 << 1, 'Start on new log file'),
         'TABLES': (1 << 2, 'close all tables'),
-        'HOSTS': (1 << 3, 'Flush host cache'),
+        'HOST': (1 << 3, 'Flush host cache'),
         'STATUS': (1 << 4, 'Flush status variables'),
         'THREADS': (1 << 5, 'Flush thread cache'),
-        'SLAVE': (1 << 6, 'Reset master info and restart slave thread'),
+        'REPLICA': (1 << 6, 'Reset source info and restart replica thread'),
+        'SLAVE': (1 << 6, 'Deprecated option; use REPLICA instead.'),
     }
 
 
